/*
  OWON PDS5022S driver

  2010-03-27
  T. Nakagawa
*/


#include <stdio.h>
#include <string.h>
#include <usb.h>
#include "owon.h"


#define VENDOR_ID 0x5345
#define PRODUCT_ID 0x1234
#define CMD_SIZE 256
#define ENDPOINT_WRITE 0x03
#define ENDPOINT_READ 0x81
#define CMD_START "START"
#define CMD_TIMEOUT 3000
#define READ_TIMEOUT 5000


usb_dev_handle *owon_new(void) {
  struct usb_bus *bus;
  struct usb_device *dev;
  usb_dev_handle *dh;
  int rv;

  usb_init();

  usb_find_busses();
  usb_find_devices();
  for (bus = usb_get_busses(); bus != NULL; bus = bus->next) {
    for (dev = bus->devices; dev != NULL; dev = dev->next) {
      if (dev->descriptor.idVendor == VENDOR_ID && dev->descriptor.idProduct == PRODUCT_ID) {
        break;
      }
    }
    if (dev != NULL) break;
  }
  if (bus == NULL) return NULL;

  if ((dh = usb_open(dev) )== NULL) return NULL;

  usb_set_configuration(dh, dev->config->bConfigurationValue);
  usb_claim_interface(dh, dev->config->interface->altsetting->bInterfaceNumber);
  if ((rv = usb_set_altinterface(dh, dev->config->interface->altsetting->bInterfaceNumber)) < 0) return NULL;
  usb_clear_halt(dh, ENDPOINT_WRITE);
  usb_clear_halt(dh, ENDPOINT_READ);

  return dh;
}


void owon_delete(usb_dev_handle *dh) {
  usb_release_interface(dh, usb_device(dh)->config->interface->altsetting->bInterfaceNumber);

  usb_reset(dh);
  usb_close(dh);

  return;
}


int owon_get(usb_dev_handle *dh, int size, unsigned char *data) {
  int rv;
  unsigned char cmd[CMD_SIZE];
  int num;

  if ((rv = usb_bulk_write(dh, ENDPOINT_WRITE, CMD_START, strlen(CMD_START), CMD_TIMEOUT)) < 0) return -1;

  if ((rv = usb_bulk_read(dh, ENDPOINT_READ, (char *)cmd, CMD_SIZE, CMD_TIMEOUT)) < 0) return -1;

  num = cmd[0] + cmd[1] * 0x100 + cmd[2] * 0x10000 + cmd[3] * 0x1000000;
  if (num > size) return -1;

  if ((rv = usb_bulk_read(dh, ENDPOINT_READ, (char *)data, size, READ_TIMEOUT)) < 0) return -1;

  return rv;
}
