/*
  Display frequency spectrum of data obtained from PDS5022S

  2010-03-27,28
  T. Nakagawa
*/


#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <gsl/gsl_fft_complex.h>
#include <gtk/gtk.h>
#include "owon.h"


#define X 50	/* X̃Obh̊Ԋu */
#define Y 50	/* ỸObh̊Ԋu */
#define INTERVAL 1000	/* IVf[^荞ފԊu(ms) */

#define WIDTH (1 + 10 * X)	/* pl̕ */
#define HEIGHT (1 + 8 * Y)	/* pl̍ */
#define DATA_MAX 1000000


static usb_dev_handle *dh;
static GtkWidget *canvas;
static GtkWidget *statusbar;
static GdkPixmap *pixmap;
static GdkGC *gc_com, *gc_ch1, *gc_ch2, *gc_cha, *gc_chb, *gc_chc, *gc_chd;
static guint contextid;
static GtkActionGroup *actions;
static unsigned char *buf;
static double *data;


/*
  X̒P(Time-mode)
*/
static char *timebase_time_str[] = {
  "5ns",
  "10ns",
  "25ns",
  "50ns",
  "100ns",
  "250ns",
  "500ns",
  "1.0us",
  "2.5us",
  "5.0us",
  "10us",
  "25us",
  "50us",
  "100us",
  "250us",
  "500us",
  "1.0ms",
  "2.5ms",
  "5.0ms",
  "10ms",
  "25ms",
  "50ms",
  "100ms",
  "250ms",
  "500ms",
  "1.0s",
  "2.5s",
  "5.0s",
  "10s",
  "25s",
  "50s",
  "100s",
};

/*
  X̒P(Frequency-mode)
*/
char *timebase_freq_str[] = {
  "5.0GHz",
  "2.5GHz",
  "1.0GHz",
  "500MHz",
  "250MHz",
  "100MHz",
  "50MHz",
  "25MHz",
  "10MHz",
  "5.0MHz",
  "2.5MHz",
  "1.0MHz",
  "500kHz",
  "250kHz",
  "100kHz",
  "50kHz",
  "25kHz",
  "10kHz",
  "5.0kHz",
  "2.5kHz",
  "1.0kHz",
  "500Hz",
  "250Hz",
  "100Hz",
  "50Hz",
  "25Hz",
  "10Hz",
  "5.0Hz",
  "2.5Hz",
  "1.0Hz",
  "500mHz",
  "250mHz",
};

/*
  Y̒P
*/
char *vertsens_str[] = {
  "?",
  "5mV",
  "10mV",
  "20mV",
  "50mV",
  "100mV",
  "200mV",
  "500mV",
  "1V",
  "2V",
  "5V",
  "10V",
  "20V",
  "50V",
  "100V",
  "200V",
  "500V",
  "1000V",
  "2000V",
  "5000V",
};


/*
  j[
*/
static const gchar *menu_info =
  "<ui>"
  "  <menubar name='Menubar'>"
  "    <menu name='File' action='File'>"
  "      <menuitem name='Reset' action='Reset'/>"
  "      <menuitem name='Quit' action='Quit'/>"
  "    </menu>"
  "    <menu name='Mode' action='Mode'>"
  "      <menuitem name='Time' action='Time'/>"
  "      <menuitem name='Frequency' action='Frequency'/>"
  "    </menu>"
  "  </menubar>"
  "</ui>";


/*
  j[̃vOI
*/
static void quit(GtkAction *action, gpointer user_data) {
  GObject *window = G_OBJECT(user_data);
  g_object_unref(g_object_get_data(window, "ui"));
  gtk_main_quit();

  return;
}


/*
  j[USB̃Zbg
*/
static void reset(GtkAction *action, gpointer user_data) {
  if (dh != NULL) {
    owon_delete(dh);
  }
  dh = owon_new();

  return;
}


/*
  j[̍
*/
static GtkActionEntry entries[] = {
  {"File", NULL, "_File"},
  {"Reset", NULL, "_Reset", "<control>R", NULL, G_CALLBACK(reset)},
  {"Quit", GTK_STOCK_QUIT, "_Quit", "<control>Q", NULL, G_CALLBACK(quit)},
  {"Mode", NULL, "_Mode"},
};


/*
  j[̍(WI{^)
*/
static GtkRadioActionEntry radio_entries[] = {
  {"Time", NULL, "_Time", "<control>T", NULL, 0},
  {"Frequency", NULL, "_Frequency", "<control>F", NULL, 1},
};


/*
  ʂ̍XV
*/
static gboolean expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer user_data) {
  gdk_draw_drawable(widget->window, widget->style->black_gc, pixmap, 0, 0, 0, 0, -1, -1);

  return FALSE;
}


/*
  IVf[^荞݉ʂ`
*/
static gint timer_event(gpointer user_data) {
  int i;
  GdkGC *gc;
  unsigned int timebase, vertsens, probrang, timebasecom;
  int size;
  int x, y, xx, yy;
  char msg[256];
  unsigned char *p;
  unsigned int len, num;
  int tmpdata;
  int mode;
  GtkAction *action;
  double real, imag;

  msg[0] = '\0';

  /* [h擾(mode=0:ԎCmode=1:g) */
  action = gtk_action_group_get_action(actions, "Time");
  mode = gtk_radio_action_get_current_value(GTK_RADIO_ACTION(action));

  /* ʂĘg */
  gdk_draw_rectangle(pixmap, canvas->style->black_gc, TRUE, 0, 0, WIDTH, HEIGHT);
  for (i = 0; i <= 8; i++) {
    gdk_draw_line(pixmap, gc_com, 0, i * Y, WIDTH - 1, i * Y);
  }
  for (i = 0; i <= 10; i++) {
    gdk_draw_line(pixmap, gc_com, i * X, 0, i * X, HEIGHT - 1);
  }

  if (dh == NULL) {
    strcpy(msg, "Cannot connect to device");
    goto refresh;
  }

  /* IVf[^𓾂 */
  size = owon_get(dh, DATA_MAX, buf);

  if (!(size >= 3 && buf[0] == 'S' && buf[1] == 'P' && buf[2] == 'B')) {
    /* f[^s(Bitmap̏ꍇȂ) */
    owon_delete(dh);
    dh = NULL;
    strcpy(msg, "Cannot connect to device");
    goto refresh;
  }

  /* e`l` */
  timebasecom = -1;
  for (p = buf + 10, len = 0; p + 51 + 2 * 5000 <= buf + size; p += 3 + len) {
    len = p[3] + p[4] * 0x100 + p[5] * 0x10000 + p[6] * 0x1000000;
    num = p[7] + p[8] * 0x100 + p[9] * 0x10000 + p[10] * 0x1000000;

    if (len != 51 - 3 + 2 * 5000 || num != 5000) {
      strcpy(msg, "Unsupported SEC/DIV");
      goto refresh;
    }

    if (p[2] == '1') {
      gc = gc_ch1;
    } else if (p[2] == '2') {
      gc = gc_ch2;
    } else if (p[2] == 'A') {
      gc = gc_cha;
    } else if (p[2] == 'B') {
      gc = gc_chb;
    } else if (p[2] == 'C') {
      gc = gc_chc;
    } else {
      gc = gc_chd;
    }
    timebase = p[19] + p[20] * 0x100 + p[21] * 0x10000 + p[22] * 0x1000000;
    vertsens = p[27] + p[28] * 0x100 + p[29] * 0x10000 + p[30] * 0x1000000;
    probrang = p[31] + p[32] * 0x100 + p[33] * 0x10000 + p[34] * 0x1000000;
    if (timebasecom == -1) {
      timebasecom = timebase;
    } else {
      if (timebase != timebasecom) {
        strcpy(msg, "SEC/DIV mismatch");
        goto refresh;
      }
    }

    if (mode == 0) {	/* Ԏ[h */
      for (i = 0; i < 5000; i++) {
        tmpdata = p[51 + 2 * i] + p[51 + 2 * i + 1] * 0x100;
        if (tmpdata >= 32768) tmpdata -= 65536;
        data[i] = tmpdata / 100.0;
      }

      xx = yy = 0;
      for (i = 0; i < 5000; i++) {
        x = i * 10 * X / 4999;
        y = 4 * Y - (int)(data[i] * 4 * Y);
        if (i != 0) gdk_draw_line(pixmap, gc, xx, yy, x, y);
        xx = x;
        yy = y;
      }

      if (probrang > 3) probrang = 3;
      sprintf(msg + strlen(msg), "CH%c %s  ", p[2], vertsens_str[vertsens + 3 * probrang]);
    } else {	/* g[h */
      for (i = 0; i < 5000; i++) {
        tmpdata = p[51 + 2 * i] + p[51 + 2 * i + 1] * 0x100;
        if (tmpdata >= 32768) tmpdata -= 65536;
        data[2 * i + 0] = tmpdata / 100.0;
        data[2 * i + 1] = 0.0;
      }
      for (; i < 8192; i++) {
        data[2 * i + 0] = 0.0;
        data[2 * i + 1] = 0.0;
      }

      /* FFTɂϊ */
      gsl_fft_complex_radix2_forward(data, 1, 8192);

      for (i = 0; i <= 4096; i++) {
        real = data[2 * i + 0];
        imag = data[2 * i + 1];
        data[i] = sqrt(real * real + imag * imag) / (5000 / 2);
      }
      data[0] /= 2;
      data[4096] /= 2;

      xx = yy = 0;
      for (i = 0; i <= 4096; i++) {
        x = i * 10 * X / 4096;
        y = 8 * Y - (int)(data[i] * 8 * Y / 2);
        if (i != 0) gdk_draw_line(pixmap, gc, xx, yy, x, y);
        xx = x;
        yy = y;
      }

      if (probrang > 3) probrang = 3;
      sprintf(msg + strlen(msg), "CH%c %s  ", p[2], vertsens_str[vertsens + 3 * probrang]);
    }
  }
  if (timebasecom != -1) {
    if (mode == 0) {
      sprintf(msg + strlen(msg), "M %s", timebase_time_str[timebasecom]);
    } else {
      sprintf(msg + strlen(msg), "M %s", timebase_freq_str[timebasecom]);
    }
  }

refresh:
  gtk_statusbar_pop(GTK_STATUSBAR(statusbar), contextid);
  gtk_statusbar_push(GTK_STATUSBAR(statusbar), contextid, msg);

  gtk_widget_queue_draw(canvas);

  return TRUE;
}


int main(int argc, char **argv) {
  GtkWidget *window;
  GtkWidget *vbox;
  GtkUIManager *ui;
  GtkWidget *menubar;
  gint timer;
  GdkColor clr_com = {0, 0xffff, 0xffff, 0xffff};
  GdkColor clr_ch1 = {0, 0xffff, 0x0000, 0x0000};
  GdkColor clr_ch2 = {0, 0xffff, 0xffff, 0x0000};
  GdkColor clr_cha = {0, 0x0000, 0xffff, 0x0000};
  GdkColor clr_chb = {0, 0x0000, 0x0000, 0xffff};
  GdkColor clr_chc = {0, 0xffff, 0x0000, 0xffff};
  GdkColor clr_chd = {0, 0x0000, 0xffff, 0xffff};

  gtk_init(&argc, &argv);

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  vbox = gtk_vbox_new(FALSE, 2);
  gtk_container_add(GTK_CONTAINER(window), vbox);

  /* j[̔zu */
  ui = gtk_ui_manager_new();
  actions = gtk_action_group_new("menu");
  gtk_action_group_add_actions(actions, entries, sizeof(entries) / sizeof(entries[0]), window);
  gtk_action_group_add_radio_actions(actions, radio_entries, sizeof(radio_entries) / sizeof(radio_entries[0]), 0, NULL, NULL);
  gtk_ui_manager_insert_action_group(ui, actions, 0);
  gtk_ui_manager_add_ui_from_string(ui, menu_info, -1, NULL);
  gtk_window_add_accel_group(GTK_WINDOW(window), gtk_ui_manager_get_accel_group(ui));     
  g_object_set_data(G_OBJECT(window), "ui", ui);
  menubar = gtk_ui_manager_get_widget(ui, "/Menubar");
  gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);

  /* Cpl̔zu */
  canvas = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(canvas), WIDTH, HEIGHT);
  gtk_box_pack_start(GTK_BOX(vbox), canvas, FALSE, FALSE, 0);

  /* Xe[^Xo[̔zu */
  statusbar = gtk_statusbar_new();
  gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(statusbar), FALSE);
  gtk_box_pack_start(GTK_BOX(vbox), statusbar, FALSE, FALSE, 0);
  contextid = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "");
  gtk_statusbar_push(GTK_STATUSBAR(statusbar), contextid, "Initializing...");

  /* Cxgݒ */
  g_signal_connect(G_OBJECT(canvas), "expose-event", G_CALLBACK(expose_event), NULL);
  timer = gtk_timeout_add(INTERVAL, (GtkFunction)timer_event, NULL);

  gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
  gtk_widget_show_all(window);

  gdk_colormap_alloc_color(gdk_colormap_get_system(), &clr_com, FALSE, TRUE);
  gdk_colormap_alloc_color(gdk_colormap_get_system(), &clr_ch1, FALSE, TRUE);
  gdk_colormap_alloc_color(gdk_colormap_get_system(), &clr_ch2, FALSE, TRUE);
  gdk_colormap_alloc_color(gdk_colormap_get_system(), &clr_cha, FALSE, TRUE);
  gdk_colormap_alloc_color(gdk_colormap_get_system(), &clr_chb, FALSE, TRUE);
  gdk_colormap_alloc_color(gdk_colormap_get_system(), &clr_chc, FALSE, TRUE);
  gdk_colormap_alloc_color(gdk_colormap_get_system(), &clr_chd, FALSE, TRUE);
  gc_com = gdk_gc_new(canvas->window);
  gc_ch1 = gdk_gc_new(canvas->window);
  gc_ch2 = gdk_gc_new(canvas->window);
  gc_cha = gdk_gc_new(canvas->window);
  gc_chb = gdk_gc_new(canvas->window);
  gc_chc = gdk_gc_new(canvas->window);
  gc_chd = gdk_gc_new(canvas->window);
  gdk_gc_set_foreground(gc_com, &clr_com);
  gdk_gc_set_foreground(gc_ch1, &clr_ch1);
  gdk_gc_set_foreground(gc_ch2, &clr_ch2);
  gdk_gc_set_foreground(gc_cha, &clr_cha);
  gdk_gc_set_foreground(gc_chb, &clr_chb);
  gdk_gc_set_foreground(gc_chc, &clr_chc);
  gdk_gc_set_foreground(gc_chd, &clr_chd);
  gdk_gc_set_line_attributes(gc_com, 1, GDK_LINE_ON_OFF_DASH, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
  pixmap = gdk_pixmap_new(canvas->window, WIDTH, HEIGHT, -1);
  buf = malloc(sizeof(unsigned char) * DATA_MAX);
  data = malloc(sizeof(double) * 2 * 8192);
  if (buf == NULL || data == NULL) return 1;

  dh = owon_new();

  gtk_main();

  gtk_timeout_remove(timer);

  if (dh != NULL) owon_delete(dh);

  return 0;
}
